@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2004, 2007, 2008  Internet Systems Consortium, Inc. ("ISC")
+ * Copyright (C) 2004, 2007-2009, 2011-2013  Internet Systems Consortium, Inc. ("ISC")
  * Copyright (C) 1999-2001  Internet Software Consortium.
  *
  * Permission to use, copy, modify, and/or distribute this software for any
@@ -15,7 +15,7 @@
  * PERFORMANCE OF THIS SOFTWARE.
  */
 
-/* $Id: t_timers.c,v 1.26.128.2 2008/01/12 23:46:43 tbox Exp $ */
+/* $Id$ */
 
 #include <config.h>
 
@@ -63,8 +63,8 @@ static void
 tx_sde(isc_task_t *task, isc_event_t *event) {
 	isc_result_t	isc_result;
 
-	task = task;
-	event = event;
+	UNUSED(task);
+	UNUSED(event);
 
 	/*
 	 * Signal shutdown processing complete.
@@ -119,8 +119,7 @@ tx_te(isc_task_t *task, isc_event_t *event) {
 
 	isc_result = isc_time_now(&now);
 	if (isc_result == ISC_R_SUCCESS) {
-		interval.seconds = Tx_seconds;
-		interval.nanoseconds = Tx_nanoseconds;
+		isc_interval_set(&interval, Tx_seconds, Tx_nanoseconds);
 		isc_result = isc_time_add(&Tx_lasttime, &interval, &base);
 		if (isc_result != ISC_R_SUCCESS) {
 			t_info("isc_time_add failed %s\n",
@@ -134,8 +133,8 @@ tx_te(isc_task_t *task, isc_event_t *event) {
 	}
 
 	if (isc_result == ISC_R_SUCCESS) {
-		interval.seconds = Tx_FUDGE_SECONDS;
-		interval.nanoseconds = Tx_FUDGE_NANOSECONDS;
+		isc_interval_set(&interval,
+				 Tx_FUDGE_SECONDS, Tx_FUDGE_NANOSECONDS);
 		isc_result = isc_time_add(&base, &interval, &ulim);
 		if (isc_result != ISC_R_SUCCESS) {
 			t_info("isc_time_add failed %s\n",
@@ -237,7 +236,7 @@ t_timers_x(isc_timertype_t timertype, isc_time_t *expires,
 		t_info("isc_taskmgr_create failed %s\n",
 		       isc_result_totext(isc_result));
 		DESTROYLOCK(&Tx_mx);
-		isc_condition_destroy(&Tx_cv);
+		(void) isc_condition_destroy(&Tx_cv);
 		isc_mem_destroy(&mctx);
 		++Tx_nprobs;
 		return;
@@ -250,7 +249,7 @@ t_timers_x(isc_timertype_t timertype, isc_time_t *expires,
 		       isc_result_totext(isc_result));
 		isc_taskmgr_destroy(&tmgr);
 		DESTROYLOCK(&Tx_mx);
-		isc_condition_destroy(&Tx_cv);
+		(void) isc_condition_destroy(&Tx_cv);
 		isc_mem_destroy(&mctx);
 		++Tx_nprobs;
 		return;
@@ -263,7 +262,7 @@ t_timers_x(isc_timertype_t timertype, isc_time_t *expires,
 		isc_timermgr_destroy(&timermgr);
 		isc_taskmgr_destroy(&tmgr);
 		DESTROYLOCK(&Tx_mx);
-		isc_condition_destroy(&Tx_cv);
+		(void) isc_condition_destroy(&Tx_cv);
 		isc_mem_destroy(&mctx);
 		++Tx_nprobs;
 		return;
@@ -277,7 +276,7 @@ t_timers_x(isc_timertype_t timertype, isc_time_t *expires,
 		isc_timermgr_destroy(&timermgr);
 		isc_taskmgr_destroy(&tmgr);
 		DESTROYLOCK(&Tx_mx);
-		isc_condition_destroy(&Tx_cv);
+		(void) isc_condition_destroy(&Tx_cv);
 		isc_mem_destroy(&mctx);
 		++Tx_nprobs;
 		return;
@@ -291,7 +290,7 @@ t_timers_x(isc_timertype_t timertype, isc_time_t *expires,
 		isc_task_destroy(&task);
 		isc_taskmgr_destroy(&tmgr);
 		DESTROYLOCK(&Tx_mx);
-		isc_condition_destroy(&Tx_cv);
+		(void) isc_condition_destroy(&Tx_cv);
 		isc_mem_destroy(&mctx);
 		++Tx_nprobs;
 		return;
@@ -303,7 +302,7 @@ t_timers_x(isc_timertype_t timertype, isc_time_t *expires,
 		isc_task_destroy(&task);
 		isc_taskmgr_destroy(&tmgr);
 		DESTROYLOCK(&Tx_mx);
-		isc_condition_destroy(&Tx_cv);
+		(void) isc_condition_destroy(&Tx_cv);
 		isc_mem_destroy(&mctx);
 		++Tx_nprobs;
 		return;
@@ -319,7 +318,7 @@ t_timers_x(isc_timertype_t timertype, isc_time_t *expires,
 		isc_task_destroy(&task);
 		isc_taskmgr_destroy(&tmgr);
 		DESTROYLOCK(&Tx_mx);
-		isc_condition_destroy(&Tx_cv);
+		(void) isc_condition_destroy(&Tx_cv);
 		isc_mem_destroy(&mctx);
 		++Tx_nprobs;
 		return;
@@ -348,7 +347,7 @@ t_timers_x(isc_timertype_t timertype, isc_time_t *expires,
 	isc_taskmgr_destroy(&tmgr);
 	isc_timermgr_destroy(&timermgr);
 	DESTROYLOCK(&Tx_mx);
-	isc_condition_destroy(&Tx_cv);
+	(void) isc_condition_destroy(&Tx_cv);
 	isc_mem_destroy(&mctx);
 
 }
@@ -367,7 +366,7 @@ t1(void) {
 	isc_time_t	expires;
 	isc_interval_t	interval;
 
-	t_assert("isc_timer_create", 1, T_REQUIRED, a1);
+	t_assert("isc_timer_create", 1, T_REQUIRED, "%s", a1);
 
 	if (threaded) {
 		Tx_nfails	= 0;
@@ -408,7 +407,7 @@ t2(void) {
 	isc_time_t	expires;
 	isc_interval_t	interval;
 
-	t_assert("isc_timer_create", 2, T_REQUIRED, a2);
+	t_assert("isc_timer_create", 2, T_REQUIRED, "%s", a2);
 
 	if (threaded) {
 		Tx_nfails	= 0;
@@ -463,8 +462,7 @@ t3_te(isc_task_t *task, isc_event_t *event) {
 	}
 
 	if (isc_result == ISC_R_SUCCESS) {
-		interval.seconds = Tx_seconds;
-		interval.nanoseconds = Tx_nanoseconds;
+		isc_interval_set(&interval, Tx_seconds, Tx_nanoseconds);
 		isc_result = isc_time_add(&Tx_lasttime, &interval, &base);
 		if (isc_result != ISC_R_SUCCESS) {
 			t_info("isc_time_add failed %s\n",
@@ -474,8 +472,8 @@ t3_te(isc_task_t *task, isc_event_t *event) {
 	}
 
 	if (isc_result == ISC_R_SUCCESS) {
-		interval.seconds = Tx_FUDGE_SECONDS;
-		interval.nanoseconds = Tx_FUDGE_NANOSECONDS;
+		isc_interval_set(&interval,
+				 Tx_FUDGE_SECONDS, Tx_FUDGE_NANOSECONDS);
 		isc_result = isc_time_add(&base, &interval, &ulim);
 		if (isc_result != ISC_R_SUCCESS) {
 			t_info("isc_time_add failed %s\n",
@@ -534,7 +532,7 @@ t3(void) {
 	isc_time_t	expires;
 	isc_interval_t	interval;
 
-	t_assert("isc_timer_create", 3, T_REQUIRED, a3);
+	t_assert("isc_timer_create", 3, T_REQUIRED, "%s", a3);
 
 	if (threaded) {
 		Tx_nfails	= 0;
@@ -599,8 +597,7 @@ t4_te(isc_task_t *task, isc_event_t *event) {
 	}
 
 	if (isc_result == ISC_R_SUCCESS) {
-		interval.seconds = Tx_seconds;
-		interval.nanoseconds = Tx_nanoseconds;
+		isc_interval_set(&interval, Tx_seconds, Tx_nanoseconds);
 		isc_result = isc_time_add(&Tx_lasttime, &interval, &base);
 		if (isc_result != ISC_R_SUCCESS) {
 			t_info("isc_time_add failed %s\n",
@@ -610,8 +607,8 @@ t4_te(isc_task_t *task, isc_event_t *event) {
 	}
 
 	if (isc_result == ISC_R_SUCCESS) {
-		interval.seconds = Tx_FUDGE_SECONDS;
-		interval.nanoseconds = Tx_FUDGE_NANOSECONDS;
+		isc_interval_set(&interval,
+				 Tx_FUDGE_SECONDS, Tx_FUDGE_NANOSECONDS);
 		isc_result = isc_time_add(&base, &interval, &ulim);
 		if (isc_result != ISC_R_SUCCESS) {
 			t_info("isc_time_add failed %s\n",
@@ -697,7 +694,7 @@ t4(void) {
 	isc_time_t	expires;
 	isc_interval_t	interval;
 
-	t_assert("isc_timer_reset", 4, T_REQUIRED, a4);
+	t_assert("isc_timer_reset", 4, T_REQUIRED, "%s", a4);
 
 	if (threaded) {
 		Tx_nfails = 0;
@@ -776,7 +773,7 @@ t5_tick_event(isc_task_t *task, isc_event_t *event) {
 	isc_time_t	expires;
 	isc_interval_t	interval;
 
-	task = task;
+	UNUSED(task);
 
 	++T5_eventcnt;
 	t_info("t5_tick_event %d\n", T5_eventcnt);
@@ -929,7 +926,7 @@ t_timers5(void) {
 		t_info("isc_taskmgr_create failed %s\n",
 		       isc_result_totext(isc_result));
 		DESTROYLOCK(&T5_mx);
-		isc_condition_destroy(&T5_cv);
+		(void) isc_condition_destroy(&T5_cv);
 		isc_mem_destroy(&mctx);
 		return(T_UNRESOLVED);
 	}
@@ -941,7 +938,7 @@ t_timers5(void) {
 		       isc_result_totext(isc_result));
 		isc_taskmgr_destroy(&tmgr);
 		DESTROYLOCK(&T5_mx);
-		isc_condition_destroy(&T5_cv);
+		(void) isc_condition_destroy(&T5_cv);
 		isc_mem_destroy(&mctx);
 		return(T_UNRESOLVED);
 	}
@@ -954,7 +951,7 @@ t_timers5(void) {
 		isc_timermgr_destroy(&timermgr);
 		isc_taskmgr_destroy(&tmgr);
 		DESTROYLOCK(&T5_mx);
-		isc_condition_destroy(&T5_cv);
+		(void) isc_condition_destroy(&T5_cv);
 		isc_mem_destroy(&mctx);
 		return(T_UNRESOLVED);
 	}
@@ -967,7 +964,7 @@ t_timers5(void) {
 		isc_task_destroy(&T5_task1);
 		isc_taskmgr_destroy(&tmgr);
 		DESTROYLOCK(&T5_mx);
-		isc_condition_destroy(&T5_cv);
+		(void) isc_condition_destroy(&T5_cv);
 		isc_mem_destroy(&mctx);
 		return(T_UNRESOLVED);
 	}
@@ -981,7 +978,7 @@ t_timers5(void) {
 		isc_task_destroy(&T5_task1);
 		isc_taskmgr_destroy(&tmgr);
 		DESTROYLOCK(&T5_mx);
-		isc_condition_destroy(&T5_cv);
+		(void) isc_condition_destroy(&T5_cv);
 		isc_mem_destroy(&mctx);
 		return(T_UNRESOLVED);
 	}
@@ -993,7 +990,7 @@ t_timers5(void) {
 		isc_timermgr_destroy(&timermgr);
 		isc_taskmgr_destroy(&tmgr);
 		DESTROYLOCK(&T5_mx);
-		isc_condition_destroy(&T5_cv);
+		(void) isc_condition_destroy(&T5_cv);
 		isc_mem_destroy(&mctx);
 		return(T_UNRESOLVED);
 	}
@@ -1018,7 +1015,7 @@ t_timers5(void) {
 		isc_task_destroy(&T5_task2);
 		isc_taskmgr_destroy(&tmgr);
 		DESTROYLOCK(&T5_mx);
-		isc_condition_destroy(&T5_cv);
+		(void) isc_condition_destroy(&T5_cv);
 		isc_mem_destroy(&mctx);
 		return(T_UNRESOLVED);
 	}
@@ -1035,7 +1032,7 @@ t_timers5(void) {
 		isc_task_destroy(&T5_task2);
 		isc_taskmgr_destroy(&tmgr);
 		DESTROYLOCK(&T5_mx);
-		isc_condition_destroy(&T5_cv);
+		(void) isc_condition_destroy(&T5_cv);
 		isc_mem_destroy(&mctx);
 		return(T_UNRESOLVED);
 	}
@@ -1054,7 +1051,7 @@ t_timers5(void) {
 		isc_task_destroy(&T5_task2);
 		isc_taskmgr_destroy(&tmgr);
 		DESTROYLOCK(&T5_mx);
-		isc_condition_destroy(&T5_cv);
+		(void) isc_condition_destroy(&T5_cv);
 		isc_mem_destroy(&mctx);
 		++T5_nprobs;
 		return(T_UNRESOLVED);
@@ -1091,7 +1088,7 @@ t_timers5(void) {
 	isc_task_destroy(&T5_task2);
 	isc_taskmgr_destroy(&tmgr);
 	DESTROYLOCK(&T5_mx);
-	isc_condition_destroy(&T5_cv);
+	(void) isc_condition_destroy(&T5_cv);
 	isc_mem_destroy(&mctx);
 
 	result = T_UNRESOLVED;
@@ -1110,7 +1107,7 @@ static const char *a5 =
 
 static void
 t5(void) {
-	t_assert("isc_timer_reset", 5, T_REQUIRED, a5);
+	t_assert("isc_timer_reset", 5, T_REQUIRED, "%s", a5);
 
 	if (threaded)
 		t_result(t_timers5());
@@ -1119,10 +1116,18 @@ t5(void) {
 }
 
 testspec_t	T_testlist[] = {
-	{	t1,		"timer_create"		},
-	{	t2,		"timer_create"		},
-	{	t3,		"timer_create"		},
-	{	t4,		"timer_reset"		},
-	{	t5,		"timer_reset"		},
-	{	NULL,		NULL			}
+	{	(PFV) t1,		"timer_create"		},
+	{	(PFV) t2,		"timer_create"		},
+	{	(PFV) t3,		"timer_create"		},
+	{	(PFV) t4,		"timer_reset"		},
+	{	(PFV) t5,		"timer_reset"		},
+	{	(PFV) NULL,		NULL			}
 };
+
+#ifdef WIN32
+int
+main(int argc, char **argv) {
+	t_settests(T_testlist);
+	return (t_main(argc, argv));
+}
+#endif
